@@ -7,7 +7,7 @@
 -----------------------------------------------------------------------
 
 if not modules then modules = { } end modules ['xindex-cfg'] = {
-      version = 0.18,
+      version = 0.28,
       comment = "main configuration to xindex.lua",
        author = "Herbert Voss",
     copyright = "Herbert Voss",
@@ -15,7 +15,7 @@ if not modules then modules = { } end modules ['xindex-cfg'] = {
 }
 
 itemPageDelimiter = ","     -- Hello, 14
-compressPages     = true    -- something like 12--15, instaead of 12,13,14,15. the |( ... |) syntax is still valid
+compressPages     = true    -- something like 12--15, instead of 12,13,14,15. the |( ... |) syntax is still valid
 fCompress	  = true    -- 3f -> page 3, 4 and 3ff -> page 3, 4, 5
 minCompress       = 3       -- 14--17 or 
 rangeSymbol       = "--"
@@ -60,31 +60,30 @@ alphabet_lower = { --   for sorting
 }
 alphabet_upper = { -- for sorting
     { ' ' },
-    { 'A', 'Á', 'À', 'Ä', 'Å', 'Æ'},
+    { 'A', 'Á', 'À', 'Ä', 'Å', 'Æ', 'Â' },
     { 'B' },
     { 'C', 'Ç' },
     { 'D' },
-    { 'E', 'È', 'É', 'Ë' },
+    { 'E', 'È', 'É', 'Ë', 'Ê' },
     { 'F' },
     { 'G' },
     { 'H' },
-    { 'I', 'Í', 'Ì', 'Ï' },
+    { 'I', 'Í', 'Ì', 'Ï', 'Î' },
     { 'J' },
     { 'K' },
     { 'L' },
     { 'M' },
     { 'N', 'Ñ' },
-    { 'O', 'Ó', 'Ò', 'Ö', 'Ø','Œ', 'Ø' },
+    { 'O', 'Ó', 'Ò', 'Ö', 'Ø','Œ', 'Ø', 'Ô'},
     { 'P' },
     { 'Q' },
     { 'R' },
     { 'S', 'Š' },
     { 'T' },
-    { 'U', 'Ú', 'Ù', 'Ü' },
+    { 'U', 'Ú', 'Ù', 'Ü', 'Û' },
     { 'V' },
     { 'W' },
     { 'X' },
     { 'Y', 'Ý', 'Ÿ' },
     { 'Z', 'Ž' }
 }
-
