@@ -5,8 +5,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-features",
-    version       = "3.00",       --TAGVERSION
-    date          = "2019-09-13", --TAGDATE
+    version       = "3.15",       --TAGVERSION
+    date          = "2020-09-02", --TAGDATE
     description   = "luaotfload submodule / features",
     license       = "GPL v2.0",
     author        = "Hans Hagen, Khaled Hosny, Elie Roux, Philipp Gesang, Marcel Krüger",
@@ -26,8 +26,11 @@ local lpeg              = require "lpeg"
 local lpegmatch         = lpeg.match
 local P                 = lpeg.P
 local R                 = lpeg.R
+local S                 = lpeg.S
 local C                 = lpeg.C
 
+local lower             = string.lower
+
 local table             = table
 local tabletohash       = table.tohash
 local tablesort         = table.sort
@@ -66,11 +69,11 @@ local stringgsub       = string.gsub
 local stringformat     = string.format
 local stringis_empty   = string.is_empty
 
-local cmp_by_idx = function (a, b) return a.idx < b.idx end
+local function cmp_by_idx (a, b) return a.idx < b.idx end
 
 local defined_combos = 0
 
-local handle_combination = function (combo, spec)
+local function handle_combination (combo, spec)
     defined_combos = defined_combos + 1
     if not combo [1] then
         report ("both", 0, "features",
@@ -148,7 +151,7 @@ local handle_combination = function (combo, spec)
         local src = fnt.characters
         local cnt = 0
 
-        local pickchr = function (uc, unavailable)
+        local function pickchr (uc, unavailable)
             local chr = src [uc]
             if unavailable == true and basechar [uc] then
                 --- fallback mode: already known
@@ -199,7 +202,7 @@ end
 
 ---[[ begin excerpt from font-ott.lua ]]
 
-local swapped = function (h)
+local function swapped (h)
     local r = { }
     for k, v in next, h do
         r[stringgsub(v,"[^a-z0-9]","")] = k -- is already lower
@@ -239,7 +242,14 @@ local support_incomplete = tabletohash({
 --doc]]--
 
 --- (string, string) dict -> (string, string) dict
-local apply_default_features = function (speclist)
+local function apply_default_features (rawlist)
+    local speclist = {}
+    for k, v in pairs(rawlist) do
+        if type(v) == 'string' then
+            v = ({['true'] = true, ['false'] = false})[lower(v)] or v
+        end
+        speclist[k] = v
+    end
     local default_features = luaotfload.features
 
     speclist = speclist or { }
@@ -247,8 +257,8 @@ local apply_default_features = function (speclist)
 
     --- handle language tag
     local language = speclist.language
-    if language then --- already lowercase at this point
-        language = stringgsub(language, "[^a-z0-9]", "")
+    if language then
+        language = stringgsub(lower(language), "[^a-z0-9]", "")
         language = rawget(verboselanguages, language) -- srsly, rawget?
                 or (languages[language] and language)
                 or "dflt"
@@ -260,7 +270,7 @@ local apply_default_features = function (speclist)
     --- handle script tag
     local script = speclist.script
     if script then
-        script = stringgsub(script, "[^a-z0-9]","")
+        script = stringgsub(lower(script), "[^a-z0-9]","")
         script = rawget(verbosescripts, script)
               or (scripts[script] and script)
               or "dflt"
@@ -317,7 +327,7 @@ local supported = {
 }
 
 --- (string | (string * string) | bool) list -> (string * number)
-local handle_slashed = function (modifiers)
+local function handle_slashed (modifiers)
     local style, optsize
     for i=1, #modifiers do
         local mod  = modifiers[i]
@@ -338,16 +348,15 @@ end
 local extract_subfont
 do
     local eof         = P(-1)
-    local digit       = R"09"
     --- Theoretically a valid subfont address can be up to ten
-    --- digits long.
-    local sub_expr    = P"(" * C(digit^1) * P")" * eof
+    --- digits long. Additionally we allow names
+    local sub_expr    = P"(" * C((1 - S"()")^1) * P")" * eof
     local full_path   = C(P(1 - sub_expr)^1)
     extract_subfont   = full_path * sub_expr
 end
 
 --- spec -> spec
-local handle_request = function (specification)
+local function handle_request (specification)
     local request = lpegmatch(luaotfload.parsers.font_request,
                               specification.specification)
 ----inspect(request)
@@ -371,7 +380,7 @@ local handle_request = function (specification)
         local fullpath, sub = lpegmatch(extract_subfont,
                                         specification.specification)
         if fullpath and sub then
-            specification.sub  = tonumber(sub)
+            specification.sub  = tonumber(sub) or sub
             specification.name = fullpath
         else
             specification.name = specification.specification
@@ -392,15 +401,7 @@ local handle_request = function (specification)
     end
 
     features.raw = request.features or {}
-    request.features = {}
-    for k, v in pairs(features.raw) do
-        if type(v) == 'string' then
-            v = string.lower(v)
-            v = ({['true'] = true, ['false'] = false})[v] or v
-        end
-        request.features[k] = v
-    end
-    request.features = apply_default_features(request.features)
+    request.features = apply_default_features(features.raw)
 
     if name then
         specification.name     = name
@@ -428,16 +429,21 @@ local handle_request = function (specification)
     --- investigated it any further (luatex-fonts-ext), so it will
     --- just stay here.
     features.normal = normalize (request.features)
-    local subfont = tonumber (request.sub)
-    if subfont and subfont >= 0 then
-        specification.sub = subfont + 1
-    else
-        specification.sub = false
+    if features.normal.instance then
+        if features.normal.axis then
+            report("term and log", 0, "features", "instance and axis provided, instance will be ignored")
+        else
+            specification.instance = features.normal.instance
+        end
     end
+    specification.sub = request.sub or specification.sub or false
 
-    if request.features and request.features.mode
-          and fonts.readers[request.features.mode] then
-        specification.forced = request.features.mode
+    local forced_mode = request.features and request.features.mode
+    if forced_mode then
+        forced_mode = lower(forced_mode)
+        if fonts.readers[forced_mode] then
+            specification.forced = forced_mode
+        end
     end
 
     return specification
@@ -452,6 +458,58 @@ if as_script == true then --- skip the remainder of the file
     return
 end
 
+do
+    local helpers = fonts.handlers.otf.readers.helpers
+    local axistofactors = helpers.axistofactors
+    local cleanname = helpers.cleanname
+    local getaxisscale = helpers.getaxisscale
+    local function search(table, term, key_field, value_field)
+        if not table then return end
+        for i=1, #table do
+            local entry = table[i]
+            if cleanname(entry[key_field]) == term then
+                return entry[value_field]
+            end
+        end
+    end
+    function helpers.getfactors(tfmdata, instance) -- `instance` might refer to an `axis` value here
+        assert(instance == true or type(instance) == "string", "Fontloader changed interface of helpers.getfactors. This is a bug, please notify the luaotfload maintainers.")
+        local variabledata = tfmdata.variabledata
+        if not variabledata or instance == "" then return end
+        local instances = variabledata.instances
+        local axis = variabledata.axis
+        local designaxis = variabledata.designaxis
+        local segments = variabledata.segments
+        if not axis then return end
+        local factors = {}
+        if instance == true then
+            for i=1, #axis do
+                local cur = axis[i]
+                local default = cur.default
+                factors[i] = getaxisscale(segments, cur.minimum, default, cur.maximum, default)
+            end
+            return factors
+        end
+        local values = search(instances, instance, "subfamily", "values")
+        if values then
+            for i=1, #axis do
+                local cur = axis[i]
+                factors[i] = getaxisscale(segments, cur.minimum, cur.default, cur.maximum, values[i].value)
+            end
+            return factors
+        end
+        values = axistofactors(instance)
+        for i=1, #axis do
+            local cur = axis[i]
+            local default = cur.default
+            local value = cur.name and values[cur.name] or values[cur.tag]
+            value = tonumber(value) or (value and search(search(designaxis, cur.tag, "tag", "variants"), cleanname(value), "name", "value")) or default
+            factors[i] = getaxisscale(segments, cur.minimum, default, cur.maximum, value)
+        end
+        return factors
+    end
+end
+
 -- MK: Added
 function fonts.definers.analyze (spec_string, size)
     return handle_request {
@@ -570,10 +628,10 @@ local autofeatures = {
     { "!!??",  interrolig_specification, "interrobang substitutions"  },
 }
 
-local add_auto_features = function ()
+local function add_auto_features ()
     local nfeats = #autofeatures
-    logreport ("both", 5, "features",
-               "auto-installing %d feature definitions", nfeats)
+    report ("both", 5, "features",
+            "auto-installing %d feature definitions", nfeats)
     for i = 1, nfeats do
         local name, spec, desc = unpack (autofeatures [i])
         spec.description = desc
@@ -581,13 +639,137 @@ local add_auto_features = function ()
     end
 end
 
-return function ()
-    logreport = luaotfload.log.report
+luaotfload.apply_default_features = apply_default_features
 
+do
+    local function mathparaminitializer(tfmdata, value, features)
+        if not next(tfmdata.mathparameters) then return end
+        if value == 'auto' then
+            if features.script == 'math' then return end
+        end
+        tfmdata.mathparameters = {}
+    end
+    fonts.constructors.features.otf.register {
+        name = 'nomathparam',
+        description = 'Set Math parameters based on this font',
+        default = 'auto',
+        initializers = {
+            base = mathparaminitializer,
+            node = mathparaminitializer,
+          -- plug = mathparaminitializer,
+        },
+    }
+end
+
+do
+    local function mathfontdimen(tfmdata, _, value)
+        if not next(tfmdata.mathparameters) then return end
+        local parameters = tfmdata.parameters
+        local mathparameters = tfmdata.mathparameters
+        if value == 'xetex' then
+            parameters[10] = mathparameters.ScriptPercentScaleDown
+            parameters[11] = mathparameters.ScriptScriptPercentScaleDown
+            parameters[12] = mathparameters.DelimitedSubFormulaMinHeight
+            parameters[13] = mathparameters.DisplayOperatorMinHeight
+            parameters[14] = mathparameters.MathLeading
+            parameters[15] = mathparameters.AxisHeight
+            parameters[16] = mathparameters.AccentBaseHeight
+            parameters[17] = mathparameters.FlattenedAccentBaseHeight
+            parameters[18] = mathparameters.SubscriptShiftDown
+            parameters[19] = mathparameters.SubscriptTopMax
+            parameters[20] = mathparameters.SubscriptBaselineDropMin
+            parameters[21] = mathparameters.SuperscriptShiftUp
+            parameters[22] = mathparameters.SuperscriptShiftUpCramped
+            parameters[23] = mathparameters.SuperscriptBottomMin
+            parameters[24] = mathparameters.SuperscriptBaselineDropMax
+            parameters[25] = mathparameters.SubSuperscriptGapMin
+            parameters[26] = mathparameters.SuperscriptBottomMaxWithSubscript
+            parameters[27] = mathparameters.SpaceAfterScript
+            parameters[28] = mathparameters.UpperLimitGapMin
+            parameters[29] = mathparameters.UpperLimitBaselineRiseMin
+            parameters[30] = mathparameters.LowerLimitGapMin
+            parameters[31] = mathparameters.LowerLimitBaselineDropMin
+            parameters[32] = mathparameters.StackTopShiftUp
+            parameters[33] = mathparameters.StackTopDisplayStyleShiftUp
+            parameters[34] = mathparameters.StackBottomShiftDown
+            parameters[35] = mathparameters.StackBottomDisplayStyleShiftDown
+            parameters[36] = mathparameters.StackGapMin
+            parameters[37] = mathparameters.StackDisplayStyleGapMin
+            parameters[38] = mathparameters.StretchStackTopShiftUp
+            parameters[39] = mathparameters.StretchStackBottomShiftDown
+            parameters[40] = mathparameters.StretchStackGapAboveMin
+            parameters[41] = mathparameters.StretchStackGapBelowMin
+            parameters[42] = mathparameters.FractionNumeratorShiftUp
+            parameters[43] = mathparameters.FractionNumeratorDisplayStyleShiftUp
+            parameters[44] = mathparameters.FractionDenominatorShiftDown
+            parameters[45] = mathparameters.FractionDenominatorDisplayStyleShiftDown
+            parameters[46] = mathparameters.FractionNumeratorGapMin
+            parameters[47] = mathparameters.FractionNumeratorDisplayStyleGapMin
+            parameters[48] = mathparameters.FractionRuleThickness
+            parameters[49] = mathparameters.FractionDenominatorGapMin
+            parameters[50] = mathparameters.FractionDenominatorDisplayStyleGapMin
+            parameters[51] = mathparameters.SkewedFractionHorizontalGap
+            parameters[52] = mathparameters.SkewedFractionVerticalGap
+            parameters[53] = mathparameters.OverbarVerticalGap
+            parameters[54] = mathparameters.OverbarRuleThickness
+            parameters[55] = mathparameters.OverbarExtraAscender
+            parameters[56] = mathparameters.UnderbarVerticalGap
+            parameters[57] = mathparameters.UnderbarRuleThickness
+            parameters[58] = mathparameters.UnderbarExtraDescender
+            parameters[59] = mathparameters.RadicalVerticalGap
+            parameters[60] = mathparameters.RadicalDisplayStyleVerticalGap
+            parameters[61] = mathparameters.RadicalRuleThickness
+            parameters[62] = mathparameters.RadicalExtraAscender
+            parameters[63] = mathparameters.RadicalKernBeforeDegree
+            parameters[64] = mathparameters.RadicalKernAfterDegree
+            parameters[65] = mathparameters.RadicalDegreeBottomRaisePercent
+            -- parameters[66] = mathparameters.MinConnectorOverlap
+            -- parameters[67] = mathparameters.SubscriptShiftDownWithSuperscript
+            -- parameters[68] = mathparameters.FractionDelimiterSize
+            -- parameters[69] = mathparameters.FractionDelimiterDisplayStyleSize
+            -- parameters[70] = mathparameters.NoLimitSubFactor
+            -- parameters[71] = mathparameters.NoLimitSupFactor
+        elseif value == 'tex2' then
+            parameters[8] = mathparameters.FractionNumeratorDisplayStyleShiftUp
+            parameters[9] = mathparameters.FractionNumeratorShiftUp
+            parameters[10] = mathparameters.StackTopShiftUp
+            parameters[11] = mathparameters.FractionDenominatorDisplayStyleShiftDown
+            parameters[12] = mathparameters.FractionDenominatorShiftDown
+            parameters[13] = mathparameters.SuperscriptShiftUp
+            parameters[14] = mathparameters.SuperscriptShiftUp
+            parameters[15] = mathparameters.SuperscriptShiftUpCramped
+            parameters[16] = mathparameters.SubscriptShiftDown
+            parameters[17] = mathparameters.SubscriptShiftDown
+            parameters[18] = mathparameters.SuperscriptBaselineDropMax
+            parameters[19] = mathparameters.SubscriptBaselineDropMin
+            parameters[20] = mathparameters.FractionDelimiterDisplayStyleSize
+            parameters[21] = mathparameters.FractionDelimiterSize
+            parameters[22] = mathparameters.AxisHeight
+        elseif value == 'tex3' then
+            parameters[8] = mathparameters.Defa
+            parameters[9] = mathparameters.UpperLimitGapMin
+            parameters[10] = mathparameters.LowerLimitGapMin
+            parameters[11] = mathparameters.UpperLimitBaselineRiseMin
+            parameters[12] = mathparameters.LowerLimitBaselineDropMin
+            parameters[13] = 0
+        end
+    end
+    fonts.constructors.features.otf.register {
+        name = 'mathfontdimen',
+        description = 'Set fontdimen values for compatibility with other engines',
+        manipulators = {
+            base = mathfontdimen,
+          -- node = mathfontdimen,
+          -- plug = mathfontdimen,
+        },
+    }
+end
+
+return function ()
     if not fonts and fonts.handlers then
-        logreport ("log", 0, "features",
-                   "OTF mechanisms missing -- did you forget to \z
-                   load a font loader?")
+        report ("log", 0, "features",
+                "OTF mechanisms missing -- did you forget to \z
+                load a font loader?")
         return false
     end
     add_auto_features ()
