@@ -115,7 +115,12 @@ local function serialize_dom(parser, current,level, output)
     attributes["_text"] = nil
   elseif xtype == "DECL" and name =="xml" then
     -- the xml declaration attributes must be in a correct order
-    insert("<?xml version='%s' encoding='%s' ?>", attributes.version, attributes.encoding)
+    local encoding = attributes.encoding or "utf-8"
+    insert("<?xml version='%s' encoding='%s' ?>", attributes.version, encoding)
+    return output
+  elseif xtype == "CDATA" then
+    -- return content unescaped
+    insert(string.format("<![CDATA[%s]]>", text_content))
     return output
   end
 
@@ -134,12 +139,17 @@ end
 -- @return DOM_Object
 local parse = function(
   xmltext --- String to be parsed
+  ,voidElements --- hash table with void elements
  )
   local domHandler = handler.domHandler()
   ---  @type DOM_Object
   local DOM_Object = xml.xmlParser(domHandler)
   -- preserve whitespace
   DOM_Object.options.stripWS = nil
+  -- don't try to expand entities
+  DOM_Object.options.expandEntities = nil
+  local voidElements = voidElements or void
+  DOM_Object._handler.options.voidElements = voidElements
   DOM_Object:parse(xmltext)
   DOM_Object.current = DOM_Object._handler.root
   DOM_Object.__index = DOM_Object
